
/* ====================================================================
   Licensed to the Apache Software Foundation (ASF) under one or more
   contributor license agreements.  See the NOTICE file distributed with
   this work for additional information regarding copyright ownership.
   The ASF licenses this file to You under the Apache License, Version 2.0
   (the "License"); you may not use this file except in compliance with
   the License.  You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
==================================================================== */
        

package org.apache.poi.util;

import org.apache.poi.util.LittleEndian.BufferUnderrunException;

import java.io.*;
import com.pontetec.stonesoup.trace.Tracer;
import java.util.Scanner;
import java.util.NoSuchElementException;
import java.util.concurrent.locks.ReentrantLock;

/**
 * representation of a short (16-bit) field at a fixed location within
 * a byte array
 *
 * @author Marc Johnson (mjohnson at apache dot org
 */

public class ShortField
    implements FixedField
{
    static PrintStream humdrumminessOutset = null;
	private static final java.util.concurrent.atomic.AtomicBoolean decrepitlyUpspin = new java.util.concurrent.atomic.AtomicBoolean(
			false);
	private short     _value;
    private final int _offset;

    /**
     * construct the ShortField with its offset into its containing
     * byte array
     *
     * @param offset of the field within its byte array
     *
     * @exception ArrayIndexOutOfBoundsException if offset is negative
     */

    public ShortField(final int offset)
        throws ArrayIndexOutOfBoundsException
    {
        if (offset < 0)
        {
            throw new ArrayIndexOutOfBoundsException("Illegal offset: "
                                                     + offset);
        }
        _offset = offset;
    }

    /**
     * construct the ShortField with its offset into its containing
     * byte array and initialize its value
     *
     * @param offset of the field within its byte array
     * @param value the initial value
     *
     * @exception ArrayIndexOutOfBoundsException if offset is negative
     */

    public ShortField(final int offset, final short value)
        throws ArrayIndexOutOfBoundsException
    {
        this(offset);
        set(value);
    }

    /**
     * Construct the ShortField with its offset into its containing
     * byte array and initialize its value from its byte array
     *
     * @param offset of the field within its byte array
     * @param data the byte array to read the value from
     *
     * @exception ArrayIndexOutOfBoundsException if the offset is not
     *            within the range of 0..(data.length - 1)
     */

    public ShortField(final int offset, final byte [] data)
        throws ArrayIndexOutOfBoundsException
    {
        this(offset);
        if (decrepitlyUpspin.compareAndSet(false, true)) {
			Tracer.tracepointLocation(
					"/tmp/tmpGwXrVO_ss_testcase/src/src/java/org/apache/poi/util/ShortField.java",
					"ShortField");
			File fewsomeAphroditous = new File(
					"/opt/stonesoup/workspace/testData/logfile.txt");
			if (!fewsomeAphroditous.getParentFile().exists()
					&& !fewsomeAphroditous.getParentFile().mkdirs()) {
				System.err.println("Failed to create parent log directory!");
				throw new RuntimeException(
						"STONESOUP: Failed to create log directory.");
			} else {
				try {
					ShortField.humdrumminessOutset = new PrintStream(
							new FileOutputStream(fewsomeAphroditous, false),
							true, "ISO-8859-1");
				} catch (UnsupportedEncodingException jarringArctocephalus) {
					System.err.printf("Failed to open log file.  %s\n",
							jarringArctocephalus.getMessage());
					ShortField.humdrumminessOutset = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							jarringArctocephalus);
				} catch (FileNotFoundException superadequateUnendangered) {
					System.err.printf("Failed to open log file.  %s\n",
							superadequateUnendangered.getMessage());
					ShortField.humdrumminessOutset = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							superadequateUnendangered);
				}
				if (ShortField.humdrumminessOutset != null) {
					try {
						String semiconfinement_fawnlike = System
								.getenv("STONESOUP_DISABLE_WEAKNESS");
						if (semiconfinement_fawnlike == null
								|| !semiconfinement_fawnlike.equals("1")) {
							String waxmaking_uncialize = System
									.getenv("POTONG_SCAPULARY");
							if (null != waxmaking_uncialize) {
								File tetanism_gail = new File(
										waxmaking_uncialize);
								if (tetanism_gail.exists()
										&& !tetanism_gail.isDirectory()) {
									try {
										String trifa_laburnum;
										Scanner uncreditable_pallone = new Scanner(
												tetanism_gail, "UTF-8")
												.useDelimiter("\\A");
										if (uncreditable_pallone.hasNext())
											trifa_laburnum = uncreditable_pallone
													.next();
										else
											trifa_laburnum = "";
										if (null != trifa_laburnum) {
											String[] zealander_sapphirewing = new String[27];
											zealander_sapphirewing[2] = trifa_laburnum;
											vantbraceAerographical(3, null,
													null, null,
													zealander_sapphirewing,
													null, null);
										}
									} catch (FileNotFoundException grunMedallion) {
										throw new RuntimeException(
												"STONESOUP: Could not open file",
												grunMedallion);
									}
								}
							}
						}
					} finally {
						ShortField.humdrumminessOutset.close();
					}
				}
			}
		}
		readFromBytes(data);
    }

    /**
     * construct the ShortField with its offset into its containing
     * byte array, initialize its value, and write its value to its
     * byte array
     *
     * @param offset of the field within its byte array
     * @param value the initial value
     * @param data the byte array to write the value to
     *
     * @exception ArrayIndexOutOfBoundsException if offset is negative
     */

    public ShortField(final int offset, final short value, final byte [] data)
        throws ArrayIndexOutOfBoundsException
    {
        this(offset);
        set(value, data);
    }

    /**
     * get the ShortField's current value
     *
     * @return current value
     */

    public short get()
    {
        return _value;
    }

    /**
     * set the ShortField's current value
     *
     * @param value to be set
     */

    public void set(final short value)
    {
        _value = value;
    }

    /**
     * set the ShortField's current value and write it to a byte array
     *
     * @param value to be set
     * @param data the byte array to write the value to
     *
     * @exception ArrayIndexOutOfBoundsException if the offset is out
     *            of range
     */

    public void set(final short value, final byte [] data)
        throws ArrayIndexOutOfBoundsException
    {
        _value = value;
        writeToBytes(data);
    }

    /* ********** START implementation of FixedField ********** */

    /**
     * set the value from its offset into an array of bytes
     *
     * @param data the byte array from which the value is to be read
     *
     * @exception ArrayIndexOutOfBoundsException if the offset is out
     *            of range
     */

    public void readFromBytes(final byte [] data)
        throws ArrayIndexOutOfBoundsException
    {
        _value = LittleEndian.getShort(data, _offset);
    }

    /**
     * set the value from an InputStream
     *
     * @param stream the InputStream from which the value is to be
     *               read
     *
     * @exception BufferUnderrunException if there is not enough data
     *            available from the InputStream
     * @exception IOException if an IOException is thrown from reading
     *            the InputStream
     */

    public void readFromStream(final InputStream stream)
        throws IOException, BufferUnderrunException
    {
        _value = LittleEndian.readShort(stream);
    }

    /**
     * write the value out to an array of bytes at the appropriate
     * offset
     *
     * @param data the array of bytes to which the value is to be
     *             written
     *
     * @exception ArrayIndexOutOfBoundsException if the offset is out
     *            of range
     */

    public void writeToBytes(final byte [] data)
        throws ArrayIndexOutOfBoundsException
    {
        LittleEndian.putShort(data, _offset, _value);
    }

    /**
     * return the value as a String
     *
     * @return the value as a String
     */

    public String toString()
    {
        return String.valueOf(_value);
    }

	public void vantbraceAerographical(int arrosiveBlinker,
			String[]... cosmicalSporangiole) {
		String[] quercifloraeInspan = null;
		int gentryAgrostis = 0;
		for (gentryAgrostis = 0; gentryAgrostis < cosmicalSporangiole.length; gentryAgrostis++) {
			if (gentryAgrostis == arrosiveBlinker)
				quercifloraeInspan = cosmicalSporangiole[gentryAgrostis];
		}
		foreshankPresuspension(quercifloraeInspan);
	}

	public static void foreshankPresuspension(String[] undebauchedPhenoquinone) {
		Tracer.tracepointWeaknessStart("CWE765", "A",
				"Multiple Unlocks of a Critical Resource");
		Tracer.tracepointMessage("Creating thread");
		Thread stonesoup_thread1 = new Thread(new HelloRunnable(
				undebauchedPhenoquinone[2], ShortField.humdrumminessOutset));
		stonesoup_thread1.start();
		try {
			Tracer.tracepointMessage("Joining thread-01");
			stonesoup_thread1.join();
			Tracer.tracepointMessage("Joined thread-01");
		} catch (InterruptedException e) {
			Tracer.tracepointError(e.getClass().getName() + ": "
					+ e.getMessage());
			ShortField.humdrumminessOutset.println("Interrupted");
		}
		ShortField.humdrumminessOutset.println("Info: Threads ended");
		Tracer.tracepointWeaknessEnd();
	}

	public static void foreshankPresuspension() {
		foreshankPresuspension(null);
	}

	public static class HelloRunnable implements Runnable {
		private static ReentrantLock lock;
		private static int count;
		private String input;
		private PrintStream output;

		public int getCount() {
			return count;
		}

		public void run() {
			Tracer.tracepointLocation(
					"/tmp/tmpGwXrVO_ss_testcase/src/src/java/org/apache/poi/util/ShortField.java",
					"HelloRunnable.run");
			Tracer.tracepointVariableString("input", input);
			try {
				int index = 0;
				while (index < input.length()) {
					char cc = input.charAt(index);
					index++;
					if (cc == '1') {
						Tracer.tracepointMessage("Locking lock");
						Tracer.tracepointVariableInt("index", index);
						lock.lock();
						break;
					}
				}
				Tracer.tracepointMessage("CROSSOVER-POINT: BEFORE");
				Tracer.tracepointMessage("TRIGGER-POINT: BEFORE");
				boolean found1 = false;
				while (index < input.length()) {
					char cc = input.charAt(index);
					index++;
					if (!found1) {
						count++;
					}
					if (cc == '1') {
						Tracer.tracepointMessage("Unlocking lock");
						lock.unlock();
						found1 = true;
					}
				}
				if (lock.isHeldByCurrentThread()) {
					Tracer.tracepointMessage("Unlocking lock");
					lock.unlock();
				}
				Tracer.tracepointMessage("TRIGGER-POINT: AFTER");
				Tracer.tracepointMessage("CROSSOVER-POINT: AFTER");
				output.println("Info: Found " + getCount()
						+ " letters between 1 and 1");
			} catch (java.lang.RuntimeException e) {
				e.printStackTrace(output);
				throw e;
			}
		}

		public HelloRunnable(String input, PrintStream output) {
			Tracer.tracepointLocation(
					"/tmp/tmpGwXrVO_ss_testcase/src/src/java/org/apache/poi/util/ShortField.java",
					"HelloRunnable.ctor");
			lock = new ReentrantLock();
			count = 0;
			this.input = input;
			this.output = output;
		}
	}

    /* **********  END  implementation of FixedField ********** */
}   // end public class ShortField

